;--------------------------------------------------------------
;            Digital I/O source code for AT90S2313
;
; by M.Wasiro @ jp.yokogawa.com
;--------------------------------------------------------------

.include "2313def.inc"

; parameter definition
.def	rcv_command			= r17				; request code to be received from PC
.def	ans_command			= r18				; 
.def	rcv_data			= r19
.def	ans_data			= r20
.def	temp				= r21
.def	temp2				= r22
; global
.def	send_buf			= r23
.def	timer_cnt			= r24
.def	sreg_buf			= r25

; define symbols
; 1. REQUEST
.equ	REQ_BYTE_OUT_PB		= 0x01				; request for output to PORTB in BYTE mode
.equ	REQ_BYTE_OUT_PD		= 0x02				; request for output to PORTD in BYTE mode
.equ	REQ_PB0_ON			= 0x10				; request for output to PORTB in BIT OR mode
.equ	REQ_PB1_ON			= 0x11
.equ	REQ_PB2_ON			= 0x12
.equ	REQ_PB3_ON			= 0x13
.equ	REQ_PB4_ON			= 0x14
.equ	REQ_PB5_ON			= 0x15
.equ	REQ_PB6_ON			= 0x16
.equ	REQ_PB7_ON			= 0x17
.equ	REQ_PB0_OFF			= 0x20				; request for output to PORTB in BIT OR mode
.equ	REQ_PB1_OFF			= 0x21
.equ	REQ_PB2_OFF			= 0x22
.equ	REQ_PB3_OFF			= 0x23
.equ	REQ_PB4_OFF			= 0x24
.equ	REQ_PB5_OFF			= 0x25
.equ	REQ_PB6_OFF			= 0x26
.equ	REQ_PB7_OFF			= 0x27
.equ	WHO_ARE_YOU			= 0x7F				; who I am?

; 2. ANSWER
.equ	ANS_BYTE_OUT_PB		= 0x81
.equ	ANS_BYTE_OUT_PD		= 0x82				; request for output to PORTD in BYTE mode
.equ	ANS_BIT_ON_PB		= 0x83				; request for output to PORTB in BIT OR mode
.equ	ANS_BIT_OFF_PB		= 0x84
.equ	ANS_BIT_ON_PD		= 0x85
.equ	ANS_BIT_OFF_PD		= 0x86

; 3. DIP SWITCH SETTING
;                               /-543---
.equ	DSW_HART			= 0b00110000		; DIP SW 1 ON
.equ	DSW_BRAIN			= 0b00101000		; DIP SW 2 ON
.equ	DSW_HARTandBRAIN	= 0b00100000		; DIP SW 1&2 ON
.equ	DSW_FF				= 0b00011000		; DIP SW 3 ON
.equ	DSW_PROFIBUS		= 0b00010000		; DIP SW 4 ON


;----------------------------------------------------------------
; boot record
	rjmp 	RESET 								; Reset Handler
	rjmp	EXT_INT0 							; IRQ0 Handler
	rjmp	EXT_INT1 							; IRQ1 Handler
	rjmp 	TIM_CAPT1 							; Timer1 Capture Handler
	rjmp 	TIM_COMP1 							; Timer1 Compare Handler
	rjmp 	TIM_OVF1 							; Timer1 Overflow Handler
	rjmp 	TIM_OVF0 							; Timer0 Overflow Handler
	rjmp 	UART_RXC 							; UART RX Complete Handler
	rjmp 	UART_DRE 							; UDR Empty Handler
	rjmp 	UART_TXC 							; UART TX Complete Handler
	rjmp 	ANA_COMP 							; Analog Comparator Handler

; START
RESET:
	; init stack pointer
	ldi		r16, low(RAMEND)					; init stack pointer
	out		spl, r16
	; init PORTB
	ldi		r16, 0xFF
	out		DDRB, r16							; mode = output
	ldi		r16, 0x00							
	out		PORTB, r16							; no pull up
	; init PORTD
	ldi		r16, 0b11000100
	out		DDRD, r16							; mode = input
	;              /-5432--
	ldi		r16, 0b00111000						; Pull up PD5432 for Dip sw
	out		PORTD, r16
	; init UART
	; UCR
	;		       7:Rx Complete Interrupt Enable
	;		        6:Tx Complete Interrupt Enable
	;		         5:UART Data Reggister empty Interrupt Enable
	;		          4:Receiver Enable
	;		           3:Transmitter Enable
	;		            2:9Bit Characters
	;		             1:Receive Data Bit 8
	;		              0:Transmit Data Bit 8
	ldi		r16, 0b00011011
	out		UCR, r16
	; UBRR OSC = 8MHz 38400bps, 0.2%Error
	ldi		r16, 0x0C							; 12(dec)
	out		UBRR, r16
	; init Timer
	; TIMSK Timer/Counter Interrupt Mask Register
	;		       7:Timer/Counter1 Overflow Interrupt Enable
	;		        6:Timer/Counter1 Output Compare Match Interrupt Enable
	;		         5:-
	;		          4:-
	;		           3:Timer/Counter1 Input Capture Interrupt Enable
	;		            2:-
	;		             1:Timer/Counter0 Overflow Interrupt Enable
	;		              0:-
	ldi		r16, 0b00000010						; Timer1 compare match
	out		TIMSK, r16
	; TCCR0 Timer/Counter1 Control Register
	ldi		r16, 0b00000101						; base clock = CK(8M) / 1024 / 8 = 976Hz
	out		TCCR0, r16
	; TCNT0 Timer/Counter0
	ldi		r16, 0x64							; 976.5625 / 100(dec) = 9Hz
	out		TCNT0, r16

TEST:
	in		temp, PIND
	;			   /-5432--
	ldi		r16, 0b00111000
	and		temp, r16
	cpi		temp, 0b00000000					; all sw on : port pull upped
	brne	MAIN
	clr		timer_cnt
	sei
TESTING:
	cpi		timer_cnt, 0x12						; 0.5Hz = 9 / 0.5 = 18(dec) = 12h
	brne	TESTING
	cli
	ldi		temp, 0x00							; if ( PORTB = 1 )
	sbis	PORTB, 0							; 	out PORTB = 0
	ldi		temp, 0xff							; else
	out		PORTB, temp							;   out PORTB = 1
	rjmp	TEST
MAIN:
	cli
wait_recv_command:
	sbis	USR, RXC
	rjmp	wait_recv_command
	in		rcv_command, UDR
	mov		send_buf, rcv_command
	rcall	sub_uart_send						; call back
	clr		timer_cnt
	sei											; M҂^C}
wait_recv_data:
	cpi		timer_cnt, 0x5A						; 5b = 0.1Hz = 9 / 0.1 = / 90(dec)
	breq	MAIN								; ^CAEgŏ
	sbis	USR, RXC
	rjmp	wait_recv_data
	in		rcv_data, UDR						; M
	cli
	mov		send_buf, rcv_data					; call back
	rcall	sub_uart_send						; send

	; switch ( rcv_command ) {
CASE1:
	cpi		rcv_command, REQ_BYTE_OUT_PB		; case( REQ_BYTE..)
	brne	CASE2
	rcall	sub_byte_out_PB						; 	action;
	rjmp	MAIN								;   break;
CASE2:
	cpi		rcv_command, REQ_PB0_ON
	brne	CASE3
	rcall	sub_PB0_ON
	rjmp	MAIN
CASE3:
	cpi		rcv_command, REQ_PB1_ON
	brne	CASE4
	rcall	sub_PB1_ON
	rjmp	MAIN
CASE4:
	cpi		rcv_command, REQ_PB2_ON
	brne	CASE5
	rcall	sub_PB2_ON
	rjmp	MAIN
CASE5:
	cpi		rcv_command, REQ_PB3_ON
	brne	CASE6
	rcall	sub_PB3_ON
	rjmp	MAIN
CASE6:
	cpi		rcv_command, REQ_PB4_ON
	brne	CASE7
	rcall	sub_PB4_ON
	rjmp	MAIN
CASE7:
	cpi		rcv_command, REQ_PB5_ON
	brne	CASE8
	rcall	sub_PB5_ON
	rjmp	MAIN
CASE8:
	cpi		rcv_command, REQ_PB6_ON
	brne	CASE9
	rcall	sub_PB6_ON
	rjmp	MAIN
CASE9:
	cpi		rcv_command, REQ_PB7_ON
	brne	CASE10
	rcall	sub_PB7_ON
	rjmp	MAIN
CASE10:
	cpi		rcv_command, REQ_PB0_OFF
	brne	CASE11
	rcall	sub_PB0_OFF
	rjmp	MAIN
CASE11:
	cpi		rcv_command, REQ_PB1_OFF
	brne	CASE12
	rcall	sub_PB1_OFF
	rjmp	MAIN
CASE12:
	cpi		rcv_command, REQ_PB2_OFF
	brne	CASE13
	rcall	sub_PB2_OFF
	rjmp	MAIN
CASE13:
	cpi		rcv_command, REQ_PB3_OFF
	brne	CASE14
	rcall	sub_PB3_OFF
	rjmp	MAIN
CASE14:
	cpi		rcv_command, REQ_PB4_OFF
	brne	CASE15
	rcall	sub_PB4_OFF
	rjmp	MAIN
CASE15:
	cpi		rcv_command, REQ_PB5_OFF
	brne	CASE16
	rcall	sub_PB5_OFF
	rjmp	MAIN
CASE16:
	cpi		rcv_command, REQ_PB6_OFF
	brne	CASE17
	rcall	sub_PB6_OFF
	rjmp	MAIN
CASE17:
	cpi		rcv_command, REQ_PB7_OFF
	brne	CASE18
	rcall	sub_PB7_OFF
	rjmp	MAIN
CASE18:
	cpi		rcv_command, WHO_ARE_YOU			; who I am?
	brne	CASE19
	rcall	sub_who_I_am
	rjmp	MAIN
CASE19:
DEFAULT:
	rjmp	MAIN
	 	
;------------------------------------------------------------------------------
;	sub_byte_out_PB
sub_byte_out_PB:
	in		sreg_buf, SREG
	out		PORTB, rcv_data
	ldi		send_buf, 0x00
	rcall	sub_uart_send
	out		SREG, sreg_buf
	ret

;------------------------------------------------------------------------------
;	sub_PB0_ON
sub_PB0_ON:
	in		sreg_buf, SREG
	sbi		PORTB, 0
	ldi		send_buf, 0x00
	rcall	sub_uart_send
	out		SREG, sreg_buf
	ret
;------------------------------------------------------------------------------
;	sub_PB1_ON
sub_PB1_ON:
	in		sreg_buf, SREG
	sbi		PORTB, 1
	ldi		send_buf, 0x00
	rcall	sub_uart_send
	out		SREG, sreg_buf
	ret
;------------------------------------------------------------------------------
;	sub_PB2_ON
sub_PB2_ON:
	in		sreg_buf, SREG
	sbi		PORTB, 2
	ldi		send_buf, 0x00
	rcall	sub_uart_send
	out		SREG, sreg_buf
	ret
;------------------------------------------------------------------------------
;	sub_PB3_ON
sub_PB3_ON:
	in		sreg_buf, SREG
	sbi		PORTB, 3
	ldi		send_buf, 0x00
	rcall	sub_uart_send
	out		SREG, sreg_buf
	ret
;------------------------------------------------------------------------------
;	sub_PB4_ON
sub_PB4_ON:
	in		sreg_buf, SREG
	sbi		PORTB, 4
	ldi		send_buf, 0x00
	rcall	sub_uart_send
	out		SREG, sreg_buf
	ret
;------------------------------------------------------------------------------
;	sub_PB5_ON
sub_PB5_ON:
	in		sreg_buf, SREG
	sbi		PORTB, 5
	ldi		send_buf, 0x00
	rcall	sub_uart_send
	out		SREG, sreg_buf
	ret
;------------------------------------------------------------------------------
;	sub_PB6_ON
sub_PB6_ON:
	in		sreg_buf, SREG
	sbi		PORTB, 6
	ldi		send_buf, 0x00
	rcall	sub_uart_send
	out		SREG, sreg_buf
	ret
;------------------------------------------------------------------------------
;	sub_PB7_ON
sub_PB7_ON:
	in		sreg_buf, SREG
	sbi		PORTB, 7
	ldi		send_buf, 0x00
	rcall	sub_uart_send
	out		SREG, sreg_buf
	ret

;------------------------------------------------------------------------------
;	sub_PB0_OFF
sub_PB0_OFF:
	in		sreg_buf, SREG
	cbi		PORTB, 0
	ldi		send_buf, 0x00
	rcall	sub_uart_send
	out		SREG, sreg_buf
	ret
;------------------------------------------------------------------------------
;	sub_PB1_OFF
sub_PB1_OFF:
	in		sreg_buf, SREG
	cbi		PORTB, 1
	ldi		send_buf, 0x00
	rcall	sub_uart_send
	out		SREG, sreg_buf
	ret
;------------------------------------------------------------------------------
;	sub_PB2_OFF
sub_PB2_OFF:
	in		sreg_buf, SREG
	cbi		PORTB, 2
	ldi		send_buf, 0x00
	rcall	sub_uart_send
	out		SREG, sreg_buf
	ret
;------------------------------------------------------------------------------
;	sub_PB3_OFF
sub_PB3_OFF:
	in		sreg_buf, SREG
	cbi		PORTB, 3
	ldi		send_buf, 0x00
	rcall	sub_uart_send
	out		SREG, sreg_buf
	ret
;------------------------------------------------------------------------------
;	sub_PB4_OFF
sub_PB4_OFF:
	in		sreg_buf, SREG
	cbi		PORTB, 4
	ldi		send_buf, 0x00
	rcall	sub_uart_send
	out		SREG, sreg_buf
	ret
;------------------------------------------------------------------------------
;	sub_PB5_OFF
sub_PB5_OFF:
	in		sreg_buf, SREG
	cbi		PORTB, 5
	ldi		send_buf, 0x00
	rcall	sub_uart_send
	out		SREG, sreg_buf
	ret
;------------------------------------------------------------------------------
;	sub_PB6_OFF
sub_PB6_OFF:
	in		sreg_buf, SREG
	cbi		PORTB, 6
	ldi		send_buf, 0x00
	rcall	sub_uart_send
	out		SREG, sreg_buf
	ret
;------------------------------------------------------------------------------
;	sub_PB7_OFF
sub_PB7_OFF:
	in		sreg_buf, SREG
	cbi		PORTB, 7
	ldi		send_buf, 0x00
	rcall	sub_uart_send
	out		SREG, sreg_buf
	ret

;------------------------------------------------------------------------------
;	sub_who_I_am
sub_who_I_am:
	in		sreg_buf, SREG
	in		temp, PIND
	ldi		temp2, 0b00111000
	and		temp, temp2
	cpi		temp, DSW_HART
	breq	i_am_hart
	cpi		temp, DSW_BRAIN
	breq	i_am_brain
	cpi		temp, DSW_HARTandBRAIN
	breq	i_am_hart_brain
	cpi		temp, DSW_FF
	breq	i_am_FF
	cpi		temp, DSW_PROFIBUS
	breq	i_am_PROFI
	ldi		send_buf, 'X'
	rjmp	answer_i_am
i_am_hart:
	ldi		send_buf, 'H'
	rjmp	answer_i_am
i_am_brain:
	ldi		send_buf, 'B'
	rjmp	answer_i_am
i_am_hart_brain:
	ldi		send_buf, 'W'
	rjmp	answer_i_am
i_am_FF:
	ldi		send_buf, 'F'
	rjmp	answer_i_am
i_am_PROFI:
	ldi		send_buf, 'P'
	rjmp	answer_i_am
answer_i_am:
	rcall	sub_uart_send
	out		SREG, sreg_buf
	ret

;------------------------------------------------------------------------------
;	Send with UART
sub_uart_send:
	in		sreg_buf, SREG
	out		UDR, send_buf
uart_sending:
	sbis	USR, TXC
	rjmp	uart_sending
	out		SREG, sreg_buf
	ret

;------------------------------------------------------------------------------
; Interrupt handlers
;
;------------------------------------------------------------------------------
;	IRQ0 Handler
EXT_INT0:
	in		sreg_buf, SREG
	out		SREG, sreg_buf
	reti

;------------------------------------------------------------------------------
;	IRQ1 Handler
EXT_INT1: 
	in		sreg_buf, SREG
	out		SREG, sreg_buf
	reti

;------------------------------------------------------------------------------
;	Timer1 Capture Handler
TIM_CAPT1:
	in		sreg_buf, SREG
	out		SREG, sreg_buf
	reti

;------------------------------------------------------------------------------
; Timer1 Compare Handler
TIM_COMP1:
	in		sreg_buf, SREG
	out		SREG, sreg_buf
	reti

;------------------------------------------------------------------------------
; Timer1 Overflow Handler
TIM_OVF1:
	in		sreg_buf, SREG
	out		SREG, sreg_buf
	reti

;------------------------------------------------------------------------------
; Timer0 Overflow Handler
TIM_OVF0:
	in		sreg_buf, SREG
	inc		timer_cnt
	out		SREG, sreg_buf
	reti

;------------------------------------------------------------------------------
; UART RX Complete Handler
UART_RXC:
	in		sreg_buf, SREG
	out		SREG, sreg_buf
	reti

;------------------------------------------------------------------------------
; UDR Empty Handler
UART_DRE:
	in		sreg_buf, SREG
	out		SREG, sreg_buf
	reti

;------------------------------------------------------------------------------
; UART TX Complete Handler
UART_TXC:
	in		sreg_buf, SREG
	out		SREG, sreg_buf
	reti

;------------------------------------------------------------------------------
; Analog Comparator Handler
ANA_COMP:
	in		sreg_buf, SREG
	out		SREG, sreg_buf
	reti
